diff --git a/src/demux.c b/src/demux.c
index 50907f5..a9f062c 100644
--- a/src/demux.c
+++ b/src/demux.c
@@ -41,8 +41,10 @@ static void *demuxing_thread(void *arg)
 
         err = av_read_frame(ctx->avf, out_packet);
         if (err == AVERROR_EOF) {
-            for (int i = 0; i < ctx->avf->nb_streams; i++)
-                sp_packet_fifo_push(ctx->dst_packets[i], NULL);
+            for (int i = 0; i < ctx->avf->nb_streams; i++) {
+                if (sp_packet_fifo_nb_dests(ctx->dst_packets[i]))
+                    sp_packet_fifo_push(ctx->dst_packets[i], NULL);
+            }
 
             sp_log(ctx, SP_LOG_VERBOSE, "Stream EOF, FIFOs flushed!\n");
             err = 0;
@@ -55,7 +57,8 @@ static void *demuxing_thread(void *arg)
         }
 
         sp_log(ctx, SP_LOG_TRACE, "Sending packet from stream %i\n", out_packet->stream_index);
-        sp_packet_fifo_push(ctx->dst_packets[out_packet->stream_index], out_packet);
+        if (sp_packet_fifo_nb_dests(ctx->dst_packets[out_packet->stream_index]))
+            sp_packet_fifo_push(ctx->dst_packets[out_packet->stream_index], out_packet);
 
         av_packet_free(&out_packet);
     }
diff --git a/src/fifo_template.c b/src/fifo_template.c
index 9112940..00580c9 100644
--- a/src/fifo_template.c
+++ b/src/fifo_template.c
@@ -264,6 +264,12 @@ unlock:
     return err;
 }
 
+int RENAME(fifo_nb_dests)(AVBufferRef *src)
+{
+    SNAME *src_ctx = (SNAME *)src->data;
+    return sp_bufferlist_len(src_ctx->dests);
+}
+
 int RENAME(fifo_pop_flags)(AVBufferRef *src, TYPE **dst, FNAME flags)
 {
     int ret = 0;
diff --git a/src/include/libtxproto/fifo_frame.h b/src/include/libtxproto/fifo_frame.h
index 8a4aaef..bf6ab87 100644
--- a/src/include/libtxproto/fifo_frame.h
+++ b/src/include/libtxproto/fifo_frame.h
@@ -42,6 +42,7 @@ AVBufferRef *RENAME(fifo_ref)(AVBufferRef *src, int max_queued, FNAME block_flag
 int RENAME(fifo_is_full)(AVBufferRef *src);
 int RENAME(fifo_get_size)(AVBufferRef *src);
 int RENAME(fifo_get_max_size)(AVBufferRef *src);
+int RENAME(fifo_nb_dests)(AVBufferRef *src);
 
 /* Modify */
 void RENAME(fifo_set_max_queued)(AVBufferRef *dst, int max_queued);
diff --git a/src/include/libtxproto/fifo_packet.h b/src/include/libtxproto/fifo_packet.h
index f17fb35..4733cb6 100644
--- a/src/include/libtxproto/fifo_packet.h
+++ b/src/include/libtxproto/fifo_packet.h
@@ -42,6 +42,7 @@ AVBufferRef *RENAME(fifo_ref)(AVBufferRef *src, int max_queued, FNAME block_flag
 int RENAME(fifo_is_full)(AVBufferRef *src);
 int RENAME(fifo_get_size)(AVBufferRef *src);
 int RENAME(fifo_get_max_size)(AVBufferRef *src);
+int RENAME(fifo_nb_dests)(AVBufferRef *src);
 
 /* Modify */
 void RENAME(fifo_set_max_queued)(AVBufferRef *dst, int max_queued);
diff --git a/src/utils.h b/src/utils.h
index f5bccc4..ea60092 100644
--- a/src/utils.h
+++ b/src/utils.h
@@ -43,6 +43,7 @@ static inline void sp_event_send_eos_packets(void *ctx, SPBufferList *events, AV
     sp_eventlist_dispatch(ctx, events, SP_EVENT_ON_EOS, &tmp);
     if (tmp != 0) {
         for (int i = 0; i < nb_fifo; i++)
-            sp_packet_fifo_push(fifo[i], NULL);
+            if (sp_packet_fifo_nb_dests(fifo[i]))
+                sp_packet_fifo_push(fifo[i], NULL);
     }
 }
